@@ -1,7 +1,7 @@
 // commands.rs
 
 // *************************************************************************
-// * Copyright (C) 2018-2019 Daniel Mueller (deso@posteo.net)              *
+// * Copyright (C) 2018-2020 Daniel Mueller (deso@posteo.net)              *
 // *                                                                       *
 // * This program is free software: you can redistribute it and/or modify  *
 // * it under the terms of the GNU General Public License as published by  *
@@ -36,11 +36,9 @@ use crate::error::Error;
 use crate::pinentry;
 use crate::Result;
 
-const NITROKEY_DEFAULT_ADMIN_PIN: &str = "12345678";
-
 /// Create an `error::Error` with an error message of the format `msg: err`.
-fn get_error(msg: &'static str, err: nitrokey::CommandError) -> Error {
-  Error::CommandError(Some(msg), err)
+fn get_error(msg: &'static str, err: nitrokey::Error) -> Error {
+  Error::NitrokeyError(Some(msg), err)
 }
 
 /// Set `libnitrokey`'s log level based on the execution context's verbosity.
@@ -62,16 +60,19 @@ fn set_log_level(ctx: &mut args::ExecCtx<'_>) {
 /// Connect to any Nitrokey device and do something with it.
 fn with_device<F>(ctx: &mut args::ExecCtx<'_>, op: F) -> Result<()>
 where
-  F: FnOnce(&mut args::ExecCtx<'_>, nitrokey::DeviceWrapper) -> Result<()>,
+  F: FnOnce(&mut args::ExecCtx<'_>, nitrokey::DeviceWrapper<'_>) -> Result<()>,
 {
+  let mut manager = nitrokey::take()?;
   set_log_level(ctx);
 
   let device = match ctx.model {
-    Some(model) => nitrokey::connect_model(model.into()).map_err(|_| {
+    Some(model) => manager.connect_model(model.into()).map_err(|_| {
       let error = format!("Nitrokey {} device not found", model.as_user_facing_str());
       Error::Error(error)
     })?,
-    None => nitrokey::connect().map_err(|_| Error::from("Nitrokey device not found"))?,
+    None => manager
+      .connect()
+      .map_err(|_| Error::from("Nitrokey device not found"))?,
   };
 
   op(ctx, device)
@@ -80,8 +81,9 @@ where
 /// Connect to a Nitrokey Storage device and do something with it.
 fn with_storage_device<F>(ctx: &mut args::ExecCtx<'_>, op: F) -> Result<()>
 where
-  F: FnOnce(&mut args::ExecCtx<'_>, nitrokey::Storage) -> Result<()>,
+  F: FnOnce(&mut args::ExecCtx<'_>, nitrokey::Storage<'_>) -> Result<()>,
 {
+  let mut manager = nitrokey::take()?;
   set_log_level(ctx);
 
   if let Some(model) = ctx.model {
@@ -92,8 +94,9 @@ where
     }
   }
 
-  let device =
-    nitrokey::Storage::connect().map_err(|_| Error::from("Nitrokey Storage device not found"))?;
+  let device = manager
+    .connect_storage()
+    .map_err(|_| Error::from("Nitrokey Storage device not found"))?;
   op(ctx, device)
 }
 
@@ -101,9 +104,9 @@ where
 /// do something with it.
 fn with_password_safe<F>(ctx: &mut args::ExecCtx<'_>, mut op: F) -> Result<()>
 where
-  F: FnMut(&mut args::ExecCtx<'_>, nitrokey::PasswordSafe<'_>) -> Result<()>,
+  F: FnMut(&mut args::ExecCtx<'_>, nitrokey::PasswordSafe<'_, '_>) -> Result<()>,
 {
-  with_device(ctx, |ctx, device| {
+  with_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(pinentry::PinType::User, &device)?;
     try_with_pin_and_data(
       ctx,
@@ -125,7 +128,7 @@ where
 /// Authenticate the given device using the given PIN type and operation.
 ///
 /// If an error occurs, the error message `msg` is used.
-fn authenticate<D, A, F>(
+fn authenticate<'mgr, D, A, F>(
   ctx: &mut args::ExecCtx<'_>,
   device: D,
   pin_type: pinentry::PinType,
@@ -133,8 +136,8 @@ fn authenticate<D, A, F>(
   op: F,
 ) -> Result<A>
 where
-  D: Device,
-  F: FnMut(&mut args::ExecCtx<'_>, D, &str) -> result::Result<A, (D, nitrokey::CommandError)>,
+  D: Device<'mgr>,
+  F: FnMut(&mut args::ExecCtx<'_>, D, &str) -> result::Result<A, (D, nitrokey::Error)>,
 {
   let pin_entry = pinentry::PinEntry::from(pin_type, &device)?;
 
@@ -142,9 +145,12 @@ where
 }
 
 /// Authenticate the given device with the user PIN.
-fn authenticate_user<T>(ctx: &mut args::ExecCtx<'_>, device: T) -> Result<nitrokey::User<T>>
+fn authenticate_user<'mgr, T>(
+  ctx: &mut args::ExecCtx<'_>,
+  device: T,
+) -> Result<nitrokey::User<'mgr, T>>
 where
-  T: Device,
+  T: Device<'mgr>,
 {
   authenticate(
     ctx,
@@ -156,9 +162,12 @@ where
 }
 
 /// Authenticate the given device with the admin PIN.
-fn authenticate_admin<T>(ctx: &mut args::ExecCtx<'_>, device: T) -> Result<nitrokey::Admin<T>>
+fn authenticate_admin<'mgr, T>(
+  ctx: &mut args::ExecCtx<'_>,
+  device: T,
+) -> Result<nitrokey::Admin<'mgr, T>>
 where
-  T: Device,
+  T: Device<'mgr>,
 {
   authenticate(
     ctx,
@@ -205,7 +214,7 @@ fn try_with_pin_and_data_with_pinentry<D, F, R, E>(
 ) -> Result<R>
 where
   F: FnMut(&mut args::ExecCtx<'_>, D, &str) -> result::Result<R, (D, E)>,
-  E: error::TryInto<nitrokey::CommandError>,
+  E: error::TryInto<nitrokey::Error>,
 {
   let mut data = data;
   let mut retry = 3;
@@ -216,7 +225,7 @@ where
       Ok(result) => return Ok(result),
       Err((new_data, err)) => match err.try_into() {
         Ok(err) => match err {
-          nitrokey::CommandError::WrongPassword => {
+          nitrokey::Error::CommandError(nitrokey::CommandError::WrongPassword) => {
             pinentry::clear(pin_entry)?;
             retry -= 1;
 
@@ -245,7 +254,7 @@ fn try_with_pin_and_data<D, F, R, E>(
 ) -> Result<R>
 where
   F: FnMut(&mut args::ExecCtx<'_>, D, &str) -> result::Result<R, (D, E)>,
-  E: Into<Error> + error::TryInto<nitrokey::CommandError>,
+  E: Into<Error> + error::TryInto<nitrokey::Error>,
 {
   let pin = match pin_entry.pin_type() {
     // Ideally we would not clone here, but that would require us to
@@ -280,7 +289,7 @@ fn try_with_pin<F, E>(
 ) -> Result<()>
 where
   F: FnMut(&str) -> result::Result<(), E>,
-  E: Into<Error> + error::TryInto<nitrokey::CommandError>,
+  E: Into<Error> + error::TryInto<nitrokey::Error>,
 {
   try_with_pin_and_data(ctx, pin_entry, msg, (), |_ctx, data, pin| {
     op(pin).map_err(|err| (data, err))
@@ -324,7 +333,7 @@ fn print_storage_status(
 fn print_status(
   ctx: &mut args::ExecCtx<'_>,
   model: &'static str,
-  device: &nitrokey::DeviceWrapper,
+  device: &nitrokey::DeviceWrapper<'_>,
 ) -> Result<()> {
   let serial_number = device
     .get_serial_number()
@@ -335,15 +344,14 @@ fn print_status(
     r#"Status:
   model:             {model}
   serial number:     0x{id}
-  firmware version:  {fwv0}.{fwv1}
+  firmware version:  {fwv}
   user retry count:  {urc}
   admin retry count: {arc}"#,
     model = model,
     id = serial_number,
-    fwv0 = device.get_major_firmware_version(),
-    fwv1 = device.get_minor_firmware_version(),
-    urc = device.get_user_retry_count(),
-    arc = device.get_admin_retry_count(),
+    fwv = device.get_firmware_version()?,
+    urc = device.get_user_retry_count()?,
+    arc = device.get_admin_retry_count()?,
   )?;
 
   if let nitrokey::DeviceWrapper::Storage(device) = device {
@@ -370,7 +378,7 @@ pub fn status(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 
 /// Perform a factory reset.
 pub fn reset(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
-  with_device(ctx, |ctx, device| {
+  with_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(pinentry::PinType::Admin, &device)?;
 
     // To force the user to enter the admin PIN before performing a
@@ -387,7 +395,7 @@ pub fn reset(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
       // build_aes_key after a factory reset on Pro devices.
       // https://github.com/Nitrokey/nitrokey-pro-firmware/issues/57
       let _ = device.get_user_retry_count();
-      device.build_aes_key(NITROKEY_DEFAULT_ADMIN_PIN)
+      device.build_aes_key(nitrokey::DEFAULT_ADMIN_PIN)
     })
   })
 }
@@ -397,7 +405,7 @@ pub fn unencrypted_set(
   ctx: &mut args::ExecCtx<'_>,
   mode: args::UnencryptedVolumeMode,
 ) -> Result<()> {
-  with_storage_device(ctx, |ctx, device| {
+  with_storage_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(pinentry::PinType::Admin, &device)?;
     let mode = match mode {
       args::UnencryptedVolumeMode::ReadWrite => nitrokey::VolumeMode::ReadWrite,
@@ -419,7 +427,7 @@ pub fn unencrypted_set(
 
 /// Open the encrypted volume on the Nitrokey.
 pub fn encrypted_open(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
-  with_storage_device(ctx, |ctx, device| {
+  with_storage_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(pinentry::PinType::User, &device)?;
 
     // We may forcefully close a hidden volume, if active, so be sure to
@@ -434,7 +442,7 @@ pub fn encrypted_open(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 
 /// Close the previously opened encrypted volume.
 pub fn encrypted_close(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
-  with_storage_device(ctx, |_ctx, device| {
+  with_storage_device(ctx, |_ctx, mut device| {
     // Flush all filesystem caches to disk. We are mostly interested in
     // making sure that the encrypted volume on the Nitrokey we are
     // about to close is not closed while not all data was written to
@@ -449,7 +457,7 @@ pub fn encrypted_close(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 
 /// Create a hidden volume.
 pub fn hidden_create(ctx: &mut args::ExecCtx<'_>, slot: u8, start: u8, end: u8) -> Result<()> {
-  with_storage_device(ctx, |ctx, device| {
+  with_storage_device(ctx, |ctx, mut device| {
     let pwd_entry = pinentry::PwdEntry::from(&device)?;
     let pwd = if let Some(pwd) = &ctx.password {
       pwd
@@ -468,7 +476,7 @@ pub fn hidden_create(ctx: &mut args::ExecCtx<'_>, slot: u8, start: u8, end: u8)
 
 /// Open a hidden volume.
 pub fn hidden_open(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
-  with_storage_device(ctx, |ctx, device| {
+  with_storage_device(ctx, |ctx, mut device| {
     let pwd_entry = pinentry::PwdEntry::from(&device)?;
     let pwd = if let Some(pwd) = &ctx.password {
       pwd
@@ -491,7 +499,7 @@ pub fn hidden_open(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 
 /// Close a previously opened hidden volume.
 pub fn hidden_close(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
-  with_storage_device(ctx, |_ctx, device| {
+  with_storage_device(ctx, |_ctx, mut device| {
     unsafe { sync() };
 
     device
@@ -539,7 +547,7 @@ pub fn config_set(
   user_password: Option<bool>,
 ) -> Result<()> {
   with_device(ctx, |ctx, device| {
-    let device = authenticate_admin(ctx, device)?;
+    let mut device = authenticate_admin(ctx, device)?;
     let config = device
       .get_config()
       .map_err(|err| get_error("Could not get configuration", err))?;
@@ -557,14 +565,17 @@ pub fn config_set(
 
 /// Lock the Nitrokey device.
 pub fn lock(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
-  with_device(ctx, |_ctx, device| {
+  with_device(ctx, |_ctx, mut device| {
     device
       .lock()
       .map_err(|err| get_error("Could not lock the device", err))
   })
 }
 
-fn get_otp<T: GenerateOtp>(slot: u8, algorithm: args::OtpAlgorithm, device: &T) -> Result<String> {
+fn get_otp<T>(slot: u8, algorithm: args::OtpAlgorithm, device: &mut T) -> Result<String>
+where
+  T: GenerateOtp,
+{
   match algorithm {
     args::OtpAlgorithm::Hotp => device.get_hotp_code(slot),
     args::OtpAlgorithm::Totp => device.get_totp_code(slot),
@@ -586,7 +597,7 @@ pub fn otp_get(
   algorithm: args::OtpAlgorithm,
   time: Option<u64>,
 ) -> Result<()> {
-  with_device(ctx, |ctx, device| {
+  with_device(ctx, |ctx, mut device| {
     if algorithm == args::OtpAlgorithm::Totp {
       device
         .set_time(
@@ -602,10 +613,10 @@ pub fn otp_get(
       .get_config()
       .map_err(|err| get_error("Could not get device configuration", err))?;
     let otp = if config.user_password {
-      let user = authenticate_user(ctx, device)?;
-      get_otp(slot, algorithm, &user)
+      let mut user = authenticate_user(ctx, device)?;
+      get_otp(slot, algorithm, &mut user)
     } else {
-      get_otp(slot, algorithm, &device)
+      get_otp(slot, algorithm, &mut device)
     }?;
     println!(ctx, "{}", otp)?;
     Ok(())
@@ -670,7 +681,7 @@ pub fn otp_set(
       }
     };
     let data = nitrokey::OtpSlotData { secret, ..data };
-    let device = authenticate_admin(ctx, device)?;
+    let mut device = authenticate_admin(ctx, device)?;
     match algorithm {
       args::OtpAlgorithm::Hotp => device.write_hotp_slot(data, counter),
       args::OtpAlgorithm::Totp => device.write_totp_slot(data, time_window),
@@ -687,7 +698,7 @@ pub fn otp_clear(
   algorithm: args::OtpAlgorithm,
 ) -> Result<()> {
   with_device(ctx, |ctx, device| {
-    let device = authenticate_admin(ctx, device)?;
+    let mut device = authenticate_admin(ctx, device)?;
     match algorithm {
       args::OtpAlgorithm::Hotp => device.erase_hotp_slot(slot),
       args::OtpAlgorithm::Totp => device.erase_totp_slot(slot),
@@ -700,7 +711,7 @@ pub fn otp_clear(
 fn print_otp_status(
   ctx: &mut args::ExecCtx<'_>,
   algorithm: args::OtpAlgorithm,
-  device: &nitrokey::DeviceWrapper,
+  device: &nitrokey::DeviceWrapper<'_>,
   all: bool,
 ) -> Result<()> {
   let mut slot: u8 = 0;
@@ -717,8 +728,8 @@ fn print_otp_status(
     };
     let name = match result {
       Ok(name) => name,
-      Err(nitrokey::CommandError::InvalidSlot) => return Ok(()),
-      Err(nitrokey::CommandError::SlotNotProgrammed) => {
+      Err(nitrokey::Error::LibraryError(nitrokey::LibraryError::InvalidSlot)) => return Ok(()),
+      Err(nitrokey::Error::CommandError(nitrokey::CommandError::SlotNotProgrammed)) => {
         if all {
           "[not programmed]".to_string()
         } else {
@@ -791,7 +802,7 @@ fn choose_pin(
 
 /// Change a PIN.
 pub fn pin_set(ctx: &mut args::ExecCtx<'_>, pin_type: pinentry::PinType) -> Result<()> {
-  with_device(ctx, |ctx, device| {
+  with_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(pin_type, &device)?;
     let new_pin = choose_pin(ctx, &pin_entry, true)?;
 
@@ -814,7 +825,7 @@ pub fn pin_set(ctx: &mut args::ExecCtx<'_>, pin_type: pinentry::PinType) -> Resu
 
 /// Unblock and reset the user PIN.
 pub fn pin_unblock(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
-  with_device(ctx, |ctx, device| {
+  with_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(pinentry::PinType::User, &device)?;
     let user_pin = choose_pin(ctx, &pin_entry, false)?;
     let pin_entry = pinentry::PinEntry::from(pinentry::PinType::Admin, &device)?;
@@ -831,7 +842,7 @@ pub fn pin_unblock(ctx: &mut args::ExecCtx<'_>) -> Result<()> {
 fn print_pws_data(
   ctx: &mut args::ExecCtx<'_>,
   description: &'static str,
-  result: result::Result<String, nitrokey::CommandError>,
+  result: result::Result<String, nitrokey::Error>,
   quiet: bool,
 ) -> Result<()> {
   let value = result.map_err(|err| get_error("Could not access PWS slot", err))?;
@@ -843,9 +854,9 @@ fn print_pws_data(
   Ok(())
 }
 
-fn check_slot(pws: &nitrokey::PasswordSafe<'_>, slot: u8) -> Result<()> {
+fn check_slot(pws: &nitrokey::PasswordSafe<'_, '_>, slot: u8) -> Result<()> {
   if slot >= nitrokey::SLOT_COUNT {
-    return Err(nitrokey::CommandError::InvalidSlot.into());
+    return Err(nitrokey::Error::from(nitrokey::LibraryError::InvalidSlot).into());
   }
   let status = pws
     .get_slot_status()
@@ -855,7 +866,7 @@ fn check_slot(pws: &nitrokey::PasswordSafe<'_>, slot: u8) -> Result<()> {
   } else {
     Err(get_error(
       "Could not access PWS slot",
-      nitrokey::CommandError::SlotNotProgrammed,
+      nitrokey::CommandError::SlotNotProgrammed.into(),
     ))
   }
 }
@@ -894,7 +905,7 @@ pub fn pws_set(
   login: &str,
   password: &str,
 ) -> Result<()> {
-  with_password_safe(ctx, |_ctx, pws| {
+  with_password_safe(ctx, |_ctx, mut pws| {
     pws
       .write_slot(slot, name, login, password)
       .map_err(|err| get_error("Could not write PWS slot", err))
@@ -903,7 +914,7 @@ pub fn pws_set(
 
 /// Clear a PWS slot.
 pub fn pws_clear(ctx: &mut args::ExecCtx<'_>, slot: u8) -> Result<()> {
-  with_password_safe(ctx, |_ctx, pws| {
+  with_password_safe(ctx, |_ctx, mut pws| {
     pws
       .erase_slot(slot)
       .map_err(|err| get_error("Could not clear PWS slot", err))
@@ -912,7 +923,7 @@ pub fn pws_clear(ctx: &mut args::ExecCtx<'_>, slot: u8) -> Result<()> {
 
 fn print_pws_slot(
   ctx: &mut args::ExecCtx<'_>,
-  pws: &nitrokey::PasswordSafe<'_>,
+  pws: &nitrokey::PasswordSafe<'_, '_>,
   slot: usize,
   programmed: bool,
 ) -> Result<()> {
@@ -938,11 +949,7 @@ pub fn pws_status(ctx: &mut args::ExecCtx<'_>, all: bool) -> Result<()> {
       .get_slot_status()
       .map_err(|err| get_error("Could not read PWS slot status", err))?;
     println!(ctx, "slot\tname")?;
-    for (i, &value) in slots
-      .into_iter()
-      .enumerate()
-      .filter(|(_, &value)| all || value)
-    {
+    for (i, &value) in slots.iter().enumerate().filter(|(_, &value)| all || value) {
       print_pws_slot(ctx, &pws, i, value)?;
     }
     Ok(())
