@@ -1,7 +1,7 @@
 // pinentry.rs
 
 // *************************************************************************
-// * Copyright (C) 2017-2019 Daniel Mueller (deso@posteo.net)              *
+// * Copyright (C) 2017-2020 Daniel Mueller (deso@posteo.net)              *
 // *                                                                       *
 // * This program is free software: you can redistribute it and/or modify  *
 // * it under the terms of the GNU General Public License as published by  *
@@ -58,9 +58,9 @@ pub struct PinEntry {
 }
 
 impl PinEntry {
-  pub fn from<D>(pin_type: PinType, device: &D) -> crate::Result<Self>
+  pub fn from<'mgr, D>(pin_type: PinType, device: &D) -> crate::Result<Self>
   where
-    D: nitrokey::Device,
+    D: nitrokey::Device<'mgr>,
   {
     let model = device.get_model();
     let serial = device.get_serial_number()?;
@@ -131,9 +131,9 @@ pub struct PwdEntry {
 }
 
 impl PwdEntry {
-  pub fn from<D>(device: &D) -> crate::Result<Self>
+  pub fn from<'mgr, D>(device: &D) -> crate::Result<Self>
   where
-    D: nitrokey::Device,
+    D: nitrokey::Device<'mgr>,
   {
     let model = device.get_model();
     let serial = device.get_serial_number()?;
@@ -357,7 +357,7 @@ mod tests {
     let response = "ERR ".to_string() + error + "\n";
     let expected = error;
 
-    let error = parse_pinentry_pin(response.to_string());
+    let error = parse_pinentry_pin(response);
 
     if let Error::Error(ref e) = error.err().unwrap() {
       assert_eq!(e, &expected);
